(* ::Package:: *)

Print["Starting."]

(* E6 tensors to be loaded from elsewhere*)
(* generators *)
E6gen=Get["./E6gen.m"];
(* cubic coupling *)
E6d=Get["./E6d.m"];


(* launch sub-kernels *)
LaunchKernels[1];


Print["Launched Kernels."]


(* quartic coupling *)
E6D = ParallelTable[Sum[E6d[[i]][[j]][[n]]E6d[[n]][[k]][[l]],{n,1,27,1}],{i,1,27,1},{j,1,27,1},{k,1,27,1},{l,1,27,1}];


Print["E6D. Done."]


(* structure constants *)
E6f = ParallelTable[-I Tr[(E6gen[[i]].E6gen[[j]]-E6gen[[j]].E6gen[[i]]).E6gen[[k]]]/3,{i,1,78,1},{j,1,78,1},{k,1,78,1}];


Print["E6f. Done."]


(* quartic adjoint coupling *)
E6chi = ParallelTable[KroneckerDelta[i,j]KroneckerDelta[k,l] + KroneckerDelta[i,k]KroneckerDelta[j,l]+ KroneckerDelta[i,l]KroneckerDelta[j,k],{i,1,78,1},{j,1,78,1},{k,1,78,1},{l,1,78,1}];


Print["E6chi. Done."]


(* mixed adjoint coupling *)
E6H = ParallelTable[(E6gen[[k]].E6gen[[l]]+E6gen[[l]].E6gen[[k]])[[i]][[j]]-2/9 KroneckerDelta[k,l] KroneckerDelta[i,j],{i,1,27,1},{j,1,27,1},{k,1,78,1},{l,1,78,1}];


Print["E6H. Done."]


(* Antisymmetric 351 *)
E6A = ConstantArray[0,{27,27}];
List351=Array[Subscript[\[Chi],##] &,{351}];
n=1;
For[i=1,i<28,i++,
For[j=i+1,j<28,j++,
{E6A[[i]][[j]] =List351[[n]]/Sqrt[2],
E6A[[j]][[i]] =-List351[[n]]/Sqrt[2],
 n=n+1}]]
Clear[n];


Print["E6A. Done."]


(* Symmetric 351 *)
E6S = ConstantArray[0,{27,27}];
List351=Array[Subscript[\[Phi],##] &,{351}];

(* off-diagonal entries *)
n=1;
For[i=1,i<28,i++,
For[j=i+1,j<28,j++,
{E6S[[i]][[j]] =List351[[n]]/Sqrt[2],
E6S[[j]][[i]] =List351[[n]]/Sqrt[2],
 n=n+1}]]
Clear[n];

(* diagonal entries *)
diagonals = Array[Subscript[\[Phi],351+##] &,{27}];
E6S = E6S + DiagonalMatrix[diagonals];
Listtot = Join[List351,diagonals];

(* define new field names and a counter *)
counter = 1;
newvar = Array[Subscript[\[Psi],##] &,{27*4}];

(* apply contraction condition to the tensor *)
For[k=1,k<28,k++,{

(* derive condition *)
condition = Sum[E6d[[k]][[i]][[j]]E6S[[i]][[j]],{i,1,27,1},{j,1,27,1}];

(* extract the old fieldnames from the condition and save them to dummy variables *)
dummy = ConstantArray[0,{5}];
n=2;
For[i=1,i<Length[Listtot]+1,i++,
If[Coefficient[condition,Listtot[[i]]] >0,{dummy[[n]]=Listtot[[i]],n=n+1},If[Coefficient[condition,Listtot[[i]]] !=0,dummy[[1]]=Listtot[[i]]]]];

(* short hand notation *)
\[Beta] = (5+Sqrt[5])/20;
\[Alpha] = 1-4 \[Beta];
all = newvar[[counter]] +newvar[[counter+1]] +newvar[[counter+2]] +newvar[[counter+3]];

(* perform the transformation to new variables *)
E6S =E6S /. dummy[[1]]-> \[Alpha] all;
For[j=1,j<5,j++,E6S =E6S /. dummy[[j+1]] -> newvar[[counter+j-1]] - \[Beta] all];

(* increment counter for new variables*)
counter = counter+4;
}]

(* simplify expression *)
E6S = Simplify[E6S];

(* list with all fieldnames phi and psi *)
Listfull =Join[Listtot,newvar];
(* a copy of it *)
List351b = Listfull;
(* zero matrix *)
Listzero = ConstantArray[0,{27,27}];

(* Find out which components have vanished in E6S and delete them from List351b *)
For[i=1,i<Length[Listfull]+1,i++,
If[D[E6S,Listfull[[i]]]==Listzero,List351b=Delete[List351b,Position[List351b,Listfull[[i]]]]]]
(* List351b only contains entries of E6S *)

(* replace the names  to chi *)
List351chi = Array[Subscript[\[Chi], ##] &,{351}];
For[i=1,i<352,i++, E6S = E6S /. List351b[[i]] -> List351chi[[i]]];


Print["E6S. Done."]


(* antisymmetric tensor eq. 4.28 *)
E6kA = ParallelTable[Sum[E6gen[[i]][[j]][[n]] E6d[[n]][[k]][[l]] - E6gen[[i]][[k]][[n]] E6d[[j]][[l]][[n]],{n,1,27,1}],{i,1,78,1},{j,1,27,1},{k,1,27,1},{l,1,27,1}];

Print["E6kA. Done."]

(* symmetric tensor eq. 4.28 *)
E6kS = ParallelTable[-Sum[E6gen[[i]][[l]][[n]] E6d[[j]][[k]][[n]] ,{n,1,27,1}],{i,1,78,1},{j,1,27,1},{k,1,27,1},{l,1,27,1}];


Print["E6kS. Done."]


Save["./E6Tensors.m", {E6d,E6gen,E6D,E6f,E6chi,E6H,E6kA,E6kS,E6A,E6S}]


Print["Saved."]
